package org.greenstone.gsdl3.action;

import org.greenstone.gsdl3.core.ModuleInterface;
import org.greenstone.gsdl3.util.*;
// XML classes
import org.w3c.dom.Node; 
import org.w3c.dom.Text; 
import org.w3c.dom.Document; 
import org.w3c.dom.Element; 

import java.util.HashMap;
import java.util.Map;
//import java.util.Map;
import java.util.Iterator;
import java.io.File;

public class QueryAction extends Action {
    

    public String process (Element message) {


	// create the return page tree
	Element page = doc_.createElement(GSXML.PAGE_ELEM);
	page.setAttribute(GSXML.LANG_ATT, message.getAttribute(GSXML.LANG_ATT));
	// add the lang stuff from message
	page.appendChild(doc_.importNode(GSXML.getChildByTagName(message, GSXML.TRANSLATION_ELEM), true));
	// add the system stuff from message
	page.appendChild(doc_.importNode(GSXML.getChildByTagName(message, GSXML.CONFIGURATION_ELEM), true));

	Element request = (Element)message.getElementsByTagName(GSXML.REQUEST_ELEM).item(0);
	// query type is teh subaction
	String query_type = request.getAttribute(GSXML.SUBACTION_ATT);
	
	if (query_type.equals("")) {
	    query_type = "text";
	}

	if (query_type.equals("text")) {
	    return textQuery(page, request);
	}
	if (query_type.equals("field")) {
	    return fieldQuery(page, request);
	}
	return unknownQuery(page, request, query_type);

	
    }
    
    // we may be able to generalize this for all query types?
    protected String textQuery(Element page, Element request) {

	// chack that the stylesheet is present - cant output a page without one. we may adapt this to use unknownquery stylesheet? - or ask for one from the MR
	String stylesheet = GSFile.stylesheetFile(config_, "textquery.xsl");
	if (stylesheet==null) {
	    return GSHTML.errorPage("textquery stylesheet not found!");
	}

	// extract the params from the cgi-request, and check that we have a coll specified
	// first convert short to long names
	Element cgi_paramList = (Element)GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	cgi_.toLong(cgi_paramList);
	HashMap params = GSXML.extractParams(cgi_paramList);

	String collection = (String)params.get("collection");
	if (collection == null || collection.equals("")) {
	    return GSHTML.errorPage("textquery, need to specify a colleciton!");
	}

	// the first part of the data for the page is the cgi-params
	Element cgi_request = (Element)doc_.importNode(request, true);
	page.appendChild(cgi_request);

	// get the service info from the MR - this will probably need to be cached somehow later on. and add as a description node to the cgi-request - this doesn't work. change to add the specified value as a current attribute to the param - need to do somthing different for query

	Element mr_info_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	Element mr_info_request = doc_.createElement(GSXML.REQUEST_ELEM);
	mr_info_message.appendChild(mr_info_request);
	mr_info_request.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_DESCRIBE);

	String to = collection;
	to = GSPath.appendLink(to, "TextQuery");
	mr_info_request.setAttribute(GSXML.TO_ATT, to);

	Element mr_info_response = (Element) mr_.process(mr_info_message);
	System.err.println("describe response for service");
	System.err.println(converter_.getString(mr_info_response));
	Element description = doc_.createElement(GSXML.DESCRIPTION_ELEM);
	// just param list for now - may need content info too
	String path = GSXML.RESPONSE_ELEM;
	path = GSPath.appendLink(path, GSXML.SERVICE_ELEM);
	path = GSPath.appendLink(path, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	Node pl = GSXML.getNodeByPath(mr_info_response, path);
	if (pl ==null) {
	    System.out.println("pl is null");
	}
	else {
	    System.out.println("pl name is "+pl.getNodeName());
	
	    Element imported_paramList = (Element)doc_.importNode(pl, true);
	    // convert the name to short
	    cgi_.addShortNames(imported_paramList);

	    description.appendChild(imported_paramList);
	
	    // for each param in page_request, set the current value if present
	    //actually, prob dont need to know the default value, just which one to display - overwrite the default att?
	    
	    Element param = (Element)imported_paramList.getFirstChild();
	    while (param !=null) {
		if (param.getNodeName().equals(GSXML.PARAM_ELEM)) { // just in case
		    String name = param.getAttribute(GSXML.NAME_ATT);
		    String current = (String)params.get(name);
		    if (current !=null && !current.equals("")) {
			param.setAttribute(GSXML.DEFAULT_ATT, current);
		    }
		}
		param = (Element)param.getNextSibling();
	    }
	}
	cgi_request.appendChild(description);
	
	// check the query string
	String query = (String)params.get(GSXML.REQUEST_TYPE_QUERY);

	if (query==null|| query.equals("")) {
	    // if there is no q set, just output the search form

	    Document style_doc = converter_.getDOM(new File(stylesheet));
	    GSXSLT.absoluteIncludePaths(style_doc, config_);
	    return transformer_.transform(style_doc, page);       
	}
	
	// else do the request, output search box and results
	Element mr_query_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	Element mr_query_request = doc_.createElement(GSXML.REQUEST_ELEM);
	mr_query_message.appendChild(mr_query_request);

	mr_query_request.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_QUERY);
	mr_query_request.setAttribute(GSXML.TO_ATT, to);

	// paramList
	Element paramList = doc_.createElement(GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	mr_query_request.appendChild(paramList);

	Iterator i = params.entrySet().iterator();
	while (i.hasNext()) {
	    Map.Entry e = (Map.Entry)i.next();
	    String name = cgi_.toLong((String)e.getKey());
	    if (!name.equals("")) {
		String value = (String)e.getValue();
	    // if (!name.equals("c")) {
		Element p = doc_.createElement(GSXML.PARAM_ELEM);
		p.setAttribute(GSXML.NAME_ATT, name);
		p.setAttribute(GSXML.VALUE_ATT, value);
		paramList.appendChild(p);
		// }
	    }
	}

	// content - contains the query string
	Element query_content = doc_.createElement(GSXML.CONTENT_ELEM);
	Text query_text = doc_.createTextNode(query);
	query_content.appendChild(query_text);
	mr_query_request.appendChild(query_content);
	
	Element mr_query_response = (Element)mr_.process(mr_query_message);

	// this result is the list of docs.
	// now take that list, and get the Titles
	// for now, just create a whole new request

	// check that there are some resources - for now check the list, but later should use a numdocs metadata elem
	path = GSXML.RESPONSE_ELEM;
	path = GSPath.appendLink(path, GSXML.CONTENT_ELEM);
	path = GSPath.appendLink(path, GSXML.RESOURCE_ELEM+GSXML.LIST_MODIFIER);

	Element resource_list = (Element)GSXML.getNodeByPath(mr_query_response,
							     path); // resourceList not present if no docs found
	if (resource_list == null) {

	    Element result_response = (Element)GSXML.getChildByTagName(mr_query_response, GSXML.RESPONSE_ELEM);
	    
	    page.appendChild(doc_.importNode(result_response, true));

	    Document style_doc = converter_.getDOM(new File(stylesheet));
	    GSXSLT.absoluteIncludePaths(style_doc, config_);
	    return transformer_.transform(style_doc, page);    

	}

	// we have a doc list, so get the metadata - for now, get title.
	// can we dynamically decide what metadata to get?
	// get the Title metadata
	Element mr_metadata_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	Element mr_metadata_request = doc_.createElement(GSXML.REQUEST_ELEM);
	mr_metadata_message.appendChild(mr_metadata_request);

	mr_metadata_request.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_QUERY);
	to = GSPath.appendLink(collection, "MetadataRetrieve");
	mr_metadata_request.setAttribute(GSXML.TO_ATT, to);
	
	Element meta_content = doc_.createElement(GSXML.CONTENT_ELEM);
	mr_metadata_request.appendChild(meta_content);

	// the first part of the content is the doc list
	meta_content.appendChild(doc_.importNode(resource_list, true));

	// the second part of the content is the metadata list
	Element metadata_list = doc_.createElement(GSXML.METADATA_ELEM+GSXML.LIST_MODIFIER);
	Element title = doc_.createElement(GSXML.METADATA_ELEM);
	title.setAttribute(GSXML.NAME_ATT, "Title");
	metadata_list.appendChild(title);
	meta_content.appendChild(metadata_list);

	Element mr_metadata_response = (Element)mr_.process(mr_metadata_message);
	
	Element result_response = (Element)GSXML.getChildByTagName(mr_metadata_response, GSXML.RESPONSE_ELEM);
	
	page.appendChild(doc_.importNode(result_response, true));
	
	// process using the stylesheet
	Document style_doc = converter_.getDOM(new File(stylesheet));
	GSXSLT.absoluteIncludePaths(style_doc, config_);
	return transformer_.transform(style_doc, page);    
	
    }

    protected String fieldQuery(Element page, Element request) {
	return GSHTML.errorPage("field query not implemented");


    }
    protected String unknownQuery(Element page, Element request, String query_type) {

	return GSHTML.errorPage("unknown query subtype - "+query_type+" query not implemented");
    }
}
